﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace SecurityDemo
{
    internal class BenchmarkTask
    {
        private string pin = "421337";
        private string md5pin;
        private string sha512pin;

        private int iterations = 20;
        private string pbkdf2pin;

        public BenchmarkTask()
        {
            using (var md5 = MD5.Create())
            {
                md5pin = Convert.ToBase64String(md5.ComputeHash(Encoding.UTF8.GetBytes(pin)));
            }
            using (var sha512 = SHA512.Create())
            {
                sha512pin = Convert.ToBase64String(sha512.ComputeHash(Encoding.UTF8.GetBytes(pin)));
            }

            var salt = RandomNumberGenerator.GetBytes(16);
            using (var function = new Rfc2898DeriveBytes(Encoding.UTF8.GetBytes(pin), salt, iterations))
            {
                var hash = function.GetBytes(24);
                pbkdf2pin = $"{Convert.ToBase64String(salt)}|{iterations}|{Convert.ToBase64String(hash)}";
            }
        }

        public bool VerifyPlainPin(string input)
        {
            return input.Equals(pin);
        }

        public bool VerifyMD5Pin(string input)
        {
            using var md5 = MD5.Create();
            return Convert.ToBase64String(md5.ComputeHash(Encoding.UTF8.GetBytes(input))).Equals(md5pin);
        }

        public bool VerifySHA512Pin(string input)
        {
            using var sha512 = SHA512.Create();
            return Convert.ToBase64String(sha512.ComputeHash(Encoding.UTF8.GetBytes(input))).Equals(sha512pin);
        }

        public bool VerifyPBKDF2Pin(string input)
        {
            var parts = pbkdf2pin.Split('|');

            var origSalt = Convert.FromBase64String(parts[0]);
            var origIterations = Int32.Parse(parts[1]);
            var origHash = parts[2];

            using var function = new Rfc2898DeriveBytes(input, origSalt, origIterations);
            var hash = function.GetBytes(24);
            return (Convert.ToBase64String(hash).Equals(origHash));
        }
    }
}
