﻿using SecurityDemo;
using System.Diagnostics;
using System.Security.Cryptography;
using System.Text;

void RngNumberDemo()
{
    Console.WriteLine("Pseudorandom:");
    for(int i = 0; i < 10; i++)
    {
        Console.Write(MyRandom.GeneratePseudorandomNumber());
    };

    Console.WriteLine();
    Console.WriteLine("Random:");
    for (int i = 0; i < 10; i++)
    {
        Console.Write(MyRandom.GenerateRandomNumber());
    };
    Console.WriteLine();
}

RngNumberDemo();

void BasicHashingDemo()
{
    Console.WriteLine("Basic hashing");
    var hash1 = MyHash.HashSHA256(Encoding.UTF8.GetBytes("Ahoj"));
    var hash2 = MyHash.HashSHA512(Encoding.UTF8.GetBytes("Ahoj"));
    var hash3 = MyHash.HashSHA512(Encoding.UTF8.GetBytes("AhOj"));

    Console.WriteLine(Convert.ToBase64String(hash1));
    Console.WriteLine(Convert.ToBase64String(hash2));
    Console.WriteLine(Convert.ToBase64String(hash3));
}

BasicHashingDemo();

void HMACDemo()
{
    Console.WriteLine("HMAC");
    var key = RandomNumberGenerator.GetBytes(32);
    var hmac = MyHMAC.HMACSHA512(Encoding.UTF8.GetBytes("Ahoj"), key);

    Console.WriteLine(Convert.ToBase64String(hmac));
}

HMACDemo();

void PBKDF2Demo()
{
    Console.WriteLine("PBKDF2");
    MyPBKDF2.HashPassword("qwerty123");

    Console.WriteLine(MyPBKDF2.Validate("PV178FTW"));
    Console.WriteLine(MyPBKDF2.Validate("qwerty123"));
}

PBKDF2Demo();

void BenchmarkTaskDemo()
{
    var task = new BenchmarkTask();
    var stopwatch = new Stopwatch();

    stopwatch.Start();
    for (int i = 0; i < 1000000; i++)
    {
        if (task.VerifyPlainPin(i.ToString()))
        {
            Console.WriteLine("Hooray");
            break;
        }
    }
    stopwatch.Stop();
    Console.WriteLine($"Plaintext runtime {String.Format("{0:n}", stopwatch.ElapsedTicks)}");

    stopwatch.Start();
    for (int i = 0; i < 1000000; i++)
    {
        if (task.VerifyMD5Pin(i.ToString()))
        {
            Console.WriteLine("Hooray");
            break;
        }
    }
    stopwatch.Stop();
    Console.WriteLine($"MD5 runtime {String.Format("{0:n}", stopwatch.ElapsedTicks)}");

    stopwatch.Start();
    for (int i = 0; i < 1000000; i++)
    {
        if (task.VerifySHA512Pin(i.ToString()))
        {
            Console.WriteLine("Hooray");
            break;
        }
    }
    stopwatch.Stop();
    Console.WriteLine($"SHA512 runtime {String.Format("{0:n}", stopwatch.ElapsedTicks)}");

    stopwatch.Start();
    for (int i = 0; i < 1000000; i++)
    {
        if (task.VerifyPBKDF2Pin(i.ToString()))
        {
            Console.WriteLine("Hooray");
            break;
        }
    }
    stopwatch.Stop();
    Console.WriteLine($"PBKDF2 runtime {String.Format("{0:n}", stopwatch.ElapsedTicks)}");

}

BenchmarkTaskDemo();

void AESDemo()
{
    var aes = new MyAES();
    var key = RandomNumberGenerator.GetBytes(32);
    var iv = RandomNumberGenerator.GetBytes(16);

    var encrypted = aes.Encrypt(Encoding.UTF8.GetBytes("allyourbasearebelongtous"), key, iv);

    Console.WriteLine(Convert.ToBase64String(encrypted));

    var decrypted = aes.Decrypt(encrypted, key, iv);

    Console.WriteLine(Encoding.UTF8.GetString(decrypted));
}

AESDemo();

void RSADemo()
{
    var rsa = new MyRSA();
    byte[] publicKey = rsa.ExportPublicKey();

    var rsa2 = new MyRSA();
    const string text = "The cake is a lie";
    rsa2.ImportPublicKey(publicKey);
    var encrypted = rsa2.Encrypt(text);

    var decrypted = rsa.Decrypt(encrypted);

    Console.WriteLine(Convert.ToBase64String(encrypted));
    Console.WriteLine(Encoding.UTF8.GetString(decrypted));
}

RSADemo();